﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class CameraManager : MonoBehaviour {

	static Vector3 scrollRange = new Vector2(5, 20);
	static float moveSpeed = 20;
	static float scrollSpeed = 15;

	private Camera camera;
	private static IEnumerator moveCam;

	void Start () {
		camera = GetComponent<Camera>();
	}
	
	void Update () {

		if (Input.GetKeyDown(KeyCode.R))
			ResetGame();

		float h = Input.GetAxis("Horizontal");
		float v = Input.GetAxis("Vertical");
		Vector2 offset = new Vector2(h, v) * Time.deltaTime * moveSpeed;
		if (offset != Vector2.zero && moveCam != null) StopCoroutine(moveCam);
		transform.position += (Vector3)offset;
		float percent = transform.position.magnitude / WorldGen.worldRadius;
		if (percent > 1) {
			transform.position /= percent;
			Vector3 pos = transform.position;
			pos.z = -10f;
			transform.position = pos;
		}
		float scroll = Input.GetAxis("Mouse ScrollWheel");
		camera.orthographicSize += scroll * Time.deltaTime * scrollSpeed;
		camera.orthographicSize = Mathf.Clamp(camera.orthographicSize, scrollRange.x, scrollRange.y);
	}

	public static void MoveCamera () {
		Vector2 position = Vector2.zero;
		int ownedCount = 0;
		foreach (Node item in WorldGen.nodes) {
			if (item.owner == Node.current) {
				position += item.position;
				ownedCount++;
			}
		}
		position /= ownedCount;
		CameraManager camMan = Camera.main.GetComponent<CameraManager>();
		if (moveCam != null) camMan.StopCoroutine(moveCam);
		moveCam = camMan.MoveTo(position);
		camMan.StartCoroutine(moveCam);
	}

	IEnumerator MoveTo (Vector2 dest) {
		Vector3 end = (Vector3)dest + Vector3.back * 10;
		while (Vector3.Distance(transform.position, end) > 0.01f) {
			transform.position = Vector3.Lerp(transform.position, end, Time.deltaTime * 4f);
			yield return null;
		}
	}

	public void ResetGame () {
		Node.links = new List<Node.Link>();
        Node.current = 1;
        Node.dragMode = false;
		UnityEngine.SceneManagement.SceneManager.LoadScene(0);
	}
}