﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class Node : MonoBehaviour {
    
	public static int teamCount = 2;
	public static Score[] scores;
	public static int current = 1;
	public static bool dragMode = false;
	public static List<Link> links = new List<Link>();


	[SerializeField] Material linkMat;
	public float radius;
	public Vector2 position {
		get { return (Vector2)transform.position; }
		set { transform.position = (Vector3)value; }
	}

	public int owner = 0;
	private bool selected = false;
	LineRenderer newLine;


	void Update () {
		if (!selected) return;
        
		Vector2 mousePos = Camera.main.ScreenToWorldPoint(Input.mousePosition);
		RaycastHit2D[] hits = Physics2D.RaycastAll(position, mousePos - position, WorldGen.worldRadius * 2);
		Node hit = hits.Length > 1 ? hits[1].transform.GetComponent<Node>() : null;
		Vector2 end = mousePos;
		if (hit != null)
			end = hit.position;
		newLine.SetPosition(1, end);

		if (Input.GetMouseButtonUp(0)) {
			if (!dragMode) {
				dragMode = true;
				return;
			}
			bool validOwner = hit.owner == 0 || hit.owner == current;
			bool existing = false;
			foreach (Link item in links)
				if ((item.node1 == this && item.node2 == hit) || item.node1 == hit && item.node2 == this)
					existing = true;
			if (validOwner && hit != null && hit != this && !existing) {
				hit.SetTeam(current);
				Link newLink = new Link(this, hit, newLine);
				links.Add(newLink);

				Link[] intersected = CollidedLines(newLine);
				foreach (Link item in intersected) {
					Node n1 = item.node1;
					Node n2 = item.node2;
					bool other1 = false, other2 = false;
					item.Remove();
					foreach (Link link in links) {
						if (link.Contains(n1))
							other1 = true;
						if (link.Contains(n2))
							other2 = true;
					}
					if (!other1)
						n1.SetTeam(current);
					if (!other2)
						n2.SetTeam(current);
				}

				IncrementTeam();
			} else {
				DestroyImmediate(newLine.gameObject); 
			}
			newLine = null;
			selected = false;
            dragMode = false;
		}
	}
    
	public void SetTeam (int index) {
		scores[owner].nodes--;
        owner = index;
		scores[owner].nodes++;
		GetComponent<SpriteRenderer>().color = TeamColor(index);
	}

	public static string GenerateScores () {
		string str = string.Empty;
		for (int i = 1; i < scores.Length; i++) {
			bool playing = (current == i);
			Color color = TeamColor(i);
			string hex = ColorUtility.ToHtmlStringRGBA(color);
			string p = "<color=#" + hex + ">" + scores[i].nodes.ToString() + "</color>";
			if (playing) p = "<size=45>" + p + "</size>";
			str += p + "    ";
		}
		return str;
	}

	public void OnClick () {
		if (dragMode || owner != current) return;

		selected = true;
		GameObject lineParent = new GameObject();
		lineParent.transform.SetParent(gameObject.transform);
		newLine = lineParent.AddComponent<LineRenderer>();
		newLine.material = linkMat;
		newLine.material.color = TeamColor(owner) / 2f;
		Vector3[] positions = { position, position };
		newLine.startWidth = 0.2f;
		newLine.endWidth = newLine.startWidth;
		newLine.SetPositions(positions);
	}

	public static Color TeamColor(int index) {
		Color[] colors = {
			Color.white,
			Color.blue,
			Color.red,
			Color.cyan,
			Color.yellow,
			Color.green,
            Color.magenta
		};
		return colors[index];
	}

	static void IncrementTeam () {
		int original = current;
		do {
			current++;
			if (current > teamCount)
				current = 1;
		} while (scores[current].nodes <= 0 && current != original);
		if (current == original) {
			GameObject.Find("Canvas/Text").GetComponent<Text>().enabled = false;
			GameObject.Find("Canvas").transform.Find("Game Over").gameObject.SetActive(true);
			GameObject.Find("Canvas/Game Over/Winner").GetComponent<Text>().text =
			    "<color=#" + ColorUtility.ToHtmlStringRGBA(TeamColor(current)).ToString() + ">" +
				"Team " + current.ToString() + "</color>" + " controls the system";
		}
		GameObject.Find("Canvas/Text").GetComponent<Text>().text = GenerateScores();
		CameraManager.MoveCamera();
	}

	public class Link {
		public LineRenderer line;
		public Node node1;
		public Node node2;
        
		public Link (Node n1, Node n2, LineRenderer l) {
			node1 = n1;
			node2 = n2;
			line = l;
		}

		public void Remove () {
			DestroyImmediate(line);
			Node.links.Remove(this);
		}

		public bool ContainsEither(Node n1, Node n2) {
			return Contains(n1) || Contains(n2);
		}

		public bool Contains(Node node) {
			return node1 == node || node2 == node;
		}
	}

	public class Score {
		public int nodes = 0;
		public int links = 0;
	}






	public static bool LineSegmentsIntersection(Vector2 p1, Vector2 p2, Vector2 p3, Vector2 p4, out Vector2 intersection) {
        intersection = Vector2.zero;

        var d = (p2.x - p1.x) * (p4.y - p3.y) - (p2.y - p1.y) * (p4.x - p3.x);

        if (d == 0.0f)
            return false;

		if ((p1 == p3) || (p1 == p4) || (p2 == p3) || (p2 == p4)) return false;

        var u = ((p3.x - p1.x) * (p4.y - p3.y) - (p3.y - p1.y) * (p4.x - p3.x)) / d;
        var v = ((p3.x - p1.x) * (p2.y - p1.y) - (p3.y - p1.y) * (p2.x - p1.x)) / d;
        
        if (u < 0.0f || u > 1.0f || v < 0.0f || v > 1.0f)
            return false;

        intersection.x = p1.x + u * (p2.x - p1.x);
        intersection.y = p1.y + u * (p2.y - p1.y);
        
        return true;
    }
    
	public static Link[] CollidedLines (LineRenderer main) {
		List<Link> list = new List<Link>();
		foreach (Link item in links) {
			if (item.line == main) continue;
			Vector2 point;
			Vector2[] p = {
				main.GetPosition(0),
				main.GetPosition(1),
				item.line.GetPosition(0),
				item.line.GetPosition(1)
			};
			if (LineSegmentsIntersection(p[0], p[1], p[2], p[3], out point)) {
				list.Add(item);
			}
		}
		return list.ToArray();
	}
}
