﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class WorldGen : MonoBehaviour {

	public static float worldRadius = 15;
	public static float maxDensity = 0.2f;
	public static Vector2 nodeSizeRange = new Vector2(0.75f, 1.5f);
	public static float minDistance = 1f;

	[SerializeField] GameObject blankNode;

	public static List<Node> nodes;
    
	void Start () {

		Node.scores = new Node.Score[Node.teamCount + 1];
		for (int i = 0; i < Node.scores.Length; i++)
			Node.scores[i] = new Node.Score();

        // Generate worlds
		Generate();

        // Initialize teams
		for (int i = 1; i <= Node.teamCount; i++)
            nodes[i].GetComponent<Node>().SetTeam(i);
		Camera.main.transform.position = (Vector3)nodes[1].position + Vector3.back * 10;

		GameObject.Find("Canvas/Text").GetComponent<Text>().text = Node.GenerateScores();
	}
    
	void Generate () {
		int nodeCount = Mathf.FloorToInt(maxDensity * Mathf.PI * worldRadius * worldRadius);
		nodes = new List<Node>();
		for (int i = 0; i < nodeCount; i++) {
			Vector2 position;
			float radius;
			bool tooClose;
			int attempts = 0;
			do {
				tooClose = false;
				radius = Random.Range(nodeSizeRange.x, nodeSizeRange.y);
				position = Random.insideUnitCircle * worldRadius;
				foreach (Node item in nodes) {
					if (Vector2.Distance(item.position, position) < item.radius + radius + minDistance)
						tooClose = true;
				}
				if (tooClose) attempts++;
				if (attempts >= 100)
					return;
			} while (tooClose);
			GameObject node = Instantiate(blankNode);
			node.transform.position = position;
			node.transform.localScale = Vector3.one * radius;
			Node nodeComp = node.GetComponent<Node>();
			nodeComp.radius = radius;
			nodes.Add(nodeComp);
		}
	}
	
	void Update () {
		
	}
}
